@@ -40,14 +40,16 @@ use std::io;
 use std::path::{Path, PathBuf};
 use std::str::FromStr;
 
-use serde::{Deserialize, Serialize};
 use tracing::instrument;
 use unscanny::{Pattern, Scanner};
 use url::Url;
 
+use distribution_types::{
+    ParsedUrlError, Requirement, UnresolvedRequirement, UnresolvedRequirementSpecification,
+};
 use pep508_rs::{
-    expand_env_vars, split_scheme, strip_host, Extras, Pep508Error, Pep508ErrorSource, Requirement,
-    RequirementsTxtRequirement, Scheme, VerbatimUrl,
+    expand_env_vars, split_scheme, strip_host, Extras, Pep508Error, Pep508ErrorSource, Scheme,
+    VerbatimUrl,
 };
 #[cfg(feature = "http")]
 use uv_client::BaseClient;
@@ -57,6 +59,10 @@ use uv_fs::{normalize_url_path, Simplified};
 use uv_normalize::ExtraName;
 use uv_warnings::warn_user;
 
+pub use crate::requirement::{RequirementsTxtRequirement, RequirementsTxtRequirementError};
+
+mod requirement;
+
 /// We emit one of those for each requirements.txt entry
 enum RequirementsTxtStatement {
     /// `-r` inclusion filename
@@ -291,28 +297,34 @@ impl Display for EditableRequirement {
     }
 }
 
-/// A [Requirement] with additional metadata from the requirements.txt, currently only hashes but in
-/// the future also editable an similar information
-#[derive(Debug, Deserialize, Clone, Eq, PartialEq, Serialize)]
+/// A [Requirement] with additional metadata from the `requirements.txt`, currently only hashes but in
+/// the future also editable and similar information.
+#[derive(Debug, Clone, Eq, PartialEq, Hash)]
 pub struct RequirementEntry {
-    /// The actual PEP 508 requirement
+    /// The actual PEP 508 requirement.
     pub requirement: RequirementsTxtRequirement,
-    /// Hashes of the downloadable packages
+    /// Hashes of the downloadable packages.
     pub hashes: Vec<String>,
-    /// Editable installation, see e.g. <https://stackoverflow.com/q/35064426/3549270>
-    pub editable: bool,
 }
 
-impl Display for RequirementEntry {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        if self.editable {
-            write!(f, "-e ")?;
-        }
-        write!(f, "{}", self.requirement)?;
-        for hash in &self.hashes {
-            write!(f, " --hash {hash}")?;
-        }
-        Ok(())
+// We place the impl here instead of next to `UnresolvedRequirementSpecification` because
+// `UnresolvedRequirementSpecification` is defined in `distribution-types` and `requirements-txt`
+// depends on `distribution-types`.
+impl TryFrom<RequirementEntry> for UnresolvedRequirementSpecification {
+    type Error = Box<ParsedUrlError>;
+
+    fn try_from(value: RequirementEntry) -> Result<Self, Self::Error> {
+        Ok(Self {
+            requirement: match value.requirement {
+                RequirementsTxtRequirement::Named(named) => {
+                    UnresolvedRequirement::Named(Requirement::from_pep508(named)?)
+                }
+                RequirementsTxtRequirement::Unnamed(unnamed) => {
+                    UnresolvedRequirement::Unnamed(unnamed)
+                }
+            },
+            hashes: value.hashes,
+        })
     }
 }
 
@@ -322,7 +334,7 @@ pub struct RequirementsTxt {
     /// The actual requirements with the hashes.
     pub requirements: Vec<RequirementEntry>,
     /// Constraints included with `-c`.
-    pub constraints: Vec<Requirement>,
+    pub constraints: Vec<pep508_rs::Requirement>,
     /// Editables with `-e`.
     pub editables: Vec<EditableRequirement>,
     /// The index URL, specified with `--index-url`.
@@ -488,7 +500,7 @@ impl RequirementsTxt {
                     // _requirements_, but we don't want to support that.
                     for entry in sub_constraints.requirements {
                         match entry.requirement {
-                            RequirementsTxtRequirement::Pep508(requirement) => {
+                            RequirementsTxtRequirement::Named(requirement) => {
                                 data.constraints.push(requirement);
                             }
                             RequirementsTxtRequirement::Unnamed(_) => {
@@ -586,7 +598,6 @@ fn parse_entry(
     Ok(Some(if s.eat_if("-r") || s.eat_if("--requirement") {
         let requirements_file = parse_value(content, s, |c: char| !['\n', '\r', '#'].contains(&c))?;
         let end = s.cursor();
-        eat_trailing_line(content, s)?;
         RequirementsTxtStatement::Requirements {
             filename: requirements_file.to_string(),
             start,
@@ -595,19 +606,18 @@ fn parse_entry(
     } else if s.eat_if("-c") || s.eat_if("--constraint") {
         let constraints_file = parse_value(content, s, |c: char| !['\n', '\r', '#'].contains(&c))?;
         let end = s.cursor();
-        eat_trailing_line(content, s)?;
         RequirementsTxtStatement::Constraint {
             filename: constraints_file.to_string(),
             start,
             end,
         }
     } else if s.eat_if("-e") || s.eat_if("--editable") {
-        let path_or_url = parse_value(content, s, |c: char| !['\n', '\r'].contains(&c))?;
+        let path_or_url = parse_value(content, s, |c: char| !['\n', '\r', '#'].contains(&c))?;
         let editable_requirement = EditableRequirement::parse(path_or_url, working_dir)
             .map_err(|err| err.with_offset(start))?;
         RequirementsTxtStatement::EditableRequirement(editable_requirement)
     } else if s.eat_if("-i") || s.eat_if("--index-url") {
-        let given = parse_value(content, s, |c: char| !['\n', '\r'].contains(&c))?;
+        let given = parse_value(content, s, |c: char| !['\n', '\r', '#'].contains(&c))?;
         let expanded = expand_env_vars(given);
         let url = VerbatimUrl::parse_url(expanded.as_ref())
             .map(|url| url.with_given(given.to_owned()))
@@ -619,7 +629,7 @@ fn parse_entry(
             })?;
         RequirementsTxtStatement::IndexUrl(url)
     } else if s.eat_if("--extra-index-url") {
-        let given = parse_value(content, s, |c: char| !['\n', '\r'].contains(&c))?;
+        let given = parse_value(content, s, |c: char| !['\n', '\r', '#'].contains(&c))?;
         let expanded = expand_env_vars(given);
         let url = VerbatimUrl::parse_url(expanded.as_ref())
             .map(|url| url.with_given(given.to_owned()))
@@ -633,7 +643,7 @@ fn parse_entry(
     } else if s.eat_if("--no-index") {
         RequirementsTxtStatement::NoIndex
     } else if s.eat_if("--find-links") || s.eat_if("-f") {
-        let path_or_url = parse_value(content, s, |c: char| !['\n', '\r'].contains(&c))?;
+        let path_or_url = parse_value(content, s, |c: char| !['\n', '\r', '#'].contains(&c))?;
         let path_or_url = FindLink::parse(path_or_url, working_dir).map_err(|err| {
             RequirementsTxtParserError::Url {
                 source: err,
@@ -644,7 +654,7 @@ fn parse_entry(
         })?;
         RequirementsTxtStatement::FindLinks(path_or_url)
     } else if s.eat_if("--no-binary") {
-        let given = parse_value(content, s, |c: char| !['\n', '\r'].contains(&c))?;
+        let given = parse_value(content, s, |c: char| !['\n', '\r', '#'].contains(&c))?;
         let specifier = PackageNameSpecifier::from_str(given).map_err(|err| {
             RequirementsTxtParserError::NoBinary {
                 source: err,
@@ -655,7 +665,7 @@ fn parse_entry(
         })?;
         RequirementsTxtStatement::NoBinary(NoBinary::from_arg(specifier))
     } else if s.eat_if("--only-binary") {
-        let given = parse_value(content, s, |c: char| !['\n', '\r'].contains(&c))?;
+        let given = parse_value(content, s, |c: char| !['\n', '\r', '#'].contains(&c))?;
         let specifier = PackageNameSpecifier::from_str(given).map_err(|err| {
             RequirementsTxtParserError::NoBinary {
                 source: err,
@@ -670,7 +680,6 @@ fn parse_entry(
         RequirementsTxtStatement::RequirementEntry(RequirementEntry {
             requirement,
             hashes,
-            editable: false,
         })
     } else if let Some(char) = s.peek() {
         let (line, column) = calculate_row_column(content, s.cursor());
@@ -787,8 +796,15 @@ fn parse_requirement_and_hashes(
     }
 
     let requirement =
-        RequirementsTxtRequirement::parse(requirement, working_dir).map_err(|err| {
-            match err.message {
+        RequirementsTxtRequirement::parse(requirement, working_dir).map_err(|err| match err {
+            RequirementsTxtRequirementError::ParsedUrl(err) => {
+                RequirementsTxtParserError::ParsedUrl {
+                    source: err,
+                    start,
+                    end,
+                }
+            }
+            RequirementsTxtRequirementError::Pep508(err) => match err.message {
                 Pep508ErrorSource::String(_) | Pep508ErrorSource::UrlError(_) => {
                     RequirementsTxtParserError::Pep508 {
                         source: err,
@@ -803,13 +819,11 @@ fn parse_requirement_and_hashes(
                         end,
                     }
                 }
-            }
+            },
         })?;
 
     let hashes = if has_hashes {
-        let hashes = parse_hashes(content, s)?;
-        eat_trailing_line(content, s)?;
-        hashes
+        parse_hashes(content, s)?
     } else {
         Vec::new()
     };
@@ -941,6 +955,11 @@ pub enum RequirementsTxtParserError {
         start: usize,
         end: usize,
     },
+    ParsedUrl {
+        source: Box<ParsedUrlError>,
+        start: usize,
+        end: usize,
+    },
     Subfile {
         source: Box<RequirementsTxtFileError>,
         start: usize,
@@ -1018,6 +1037,11 @@ impl RequirementsTxtParserError {
                 start: start + offset,
                 end: end + offset,
             },
+            Self::ParsedUrl { source, start, end } => Self::ParsedUrl {
+                source,
+                start: start + offset,
+                end: end + offset,
+            },
             Self::Subfile { source, start, end } => Self::Subfile {
                 source,
                 start: start + offset,
@@ -1068,6 +1092,9 @@ impl Display for RequirementsTxtParserError {
             Self::Pep508 { start, .. } => {
                 write!(f, "Couldn't parse requirement at position {start}")
             }
+            Self::ParsedUrl { start, .. } => {
+                write!(f, "Couldn't URL at position {start}")
+            }
             Self::Subfile { start, .. } => {
                 write!(f, "Error parsing included file at position {start}")
             }
@@ -1099,6 +1126,7 @@ impl std::error::Error for RequirementsTxtParserError {
             Self::UnnamedConstraint { .. } => None,
             Self::UnsupportedRequirement { source, .. } => Some(source),
             Self::Pep508 { source, .. } => Some(source),
+            Self::ParsedUrl { source, .. } => Some(source),
             Self::Subfile { source, .. } => Some(source.as_ref()),
             Self::Parser { .. } => None,
             Self::NonUnicodeUrl { .. } => None,
@@ -1186,6 +1214,13 @@ impl Display for RequirementsTxtFileError {
                     self.file.user_display(),
                 )
             }
+            RequirementsTxtParserError::ParsedUrl { start, .. } => {
+                write!(
+                    f,
+                    "Couldn't parse URL in `{}` at position {start}",
+                    self.file.user_display(),
+                )
+            }
             RequirementsTxtParserError::Subfile { start, .. } => {
                 write!(
                     f,
@@ -1290,6 +1325,7 @@ mod test {
     use tempfile::tempdir;
     use test_case::test_case;
     use unscanny::Scanner;
+
     use uv_client::BaseClientBuilder;
     use uv_fs::Simplified;
 
@@ -1732,7 +1768,7 @@ mod test {
         RequirementsTxt {
             requirements: [
                 RequirementEntry {
-                    requirement: Pep508(
+                    requirement: Named(
                         Requirement {
                             name: PackageName(
                                 "flask",
@@ -1743,7 +1779,6 @@ mod test {
                         },
                     ),
                     hashes: [],
-                    editable: false,
                 },
             ],
             constraints: [],
@@ -1787,7 +1822,7 @@ mod test {
         RequirementsTxt {
             requirements: [
                 RequirementEntry {
-                    requirement: Pep508(
+                    requirement: Named(
                         Requirement {
                             name: PackageName(
                                 "flask",
@@ -1798,7 +1833,6 @@ mod test {
                         },
                     ),
                     hashes: [],
-                    editable: false,
                 },
             ],
             constraints: [],
@@ -1925,12 +1959,199 @@ mod test {
         insta::with_settings!({
             filters => filters
         }, {
-            insta::assert_snapshot!(errors, @"Nested `requirements` file contains conflicting `--index-url` at <REQUIREMENTS_TXT>:3:1");
+            insta::assert_snapshot!(errors, @"Nested `requirements` file contains conflicting `--index-url` at <REQUIREMENTS_TXT>:2:13");
         });
 
         Ok(())
     }
 
+    #[tokio::test]
+    async fn comments() -> Result<()> {
+        let temp_dir = assert_fs::TempDir::new()?;
+
+        let requirements_txt = temp_dir.child("requirements.txt");
+        requirements_txt.write_str(indoc! {r"
+            -r ./sibling.txt  # comment
+            --index-url https://test.pypi.org/simple/  # comment
+            --no-binary :all:  # comment
+
+            flask==3.0.0 \
+                --hash=sha256:1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef \
+                # comment
+
+            requests==2.26.0 \
+                --hash=sha256:fedcba0987654321fedcba0987654321fedcba0987654321fedcba0987654321 # comment
+
+            black==21.12b0 # comment
+
+            mypy==0.910 \
+              # comment
+        "})?;
+
+        let sibling_txt = temp_dir.child("sibling.txt");
+        sibling_txt.write_str(indoc! {"
+            httpx # comment
+        "})?;
+
+        let requirements = RequirementsTxt::parse(
+            requirements_txt.path(),
+            temp_dir.path(),
+            &BaseClientBuilder::new(),
+        )
+        .await
+        .unwrap();
+        insta::assert_debug_snapshot!(requirements, @r###"
+        RequirementsTxt {
+            requirements: [
+                RequirementEntry {
+                    requirement: Named(
+                        Requirement {
+                            name: PackageName(
+                                "httpx",
+                            ),
+                            extras: [],
+                            version_or_url: None,
+                            marker: None,
+                        },
+                    ),
+                    hashes: [],
+                },
+                RequirementEntry {
+                    requirement: Named(
+                        Requirement {
+                            name: PackageName(
+                                "flask",
+                            ),
+                            extras: [],
+                            version_or_url: Some(
+                                VersionSpecifier(
+                                    VersionSpecifiers(
+                                        [
+                                            VersionSpecifier {
+                                                operator: Equal,
+                                                version: "3.0.0",
+                                            },
+                                        ],
+                                    ),
+                                ),
+                            ),
+                            marker: None,
+                        },
+                    ),
+                    hashes: [
+                        "sha256:1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef",
+                    ],
+                },
+                RequirementEntry {
+                    requirement: Named(
+                        Requirement {
+                            name: PackageName(
+                                "requests",
+                            ),
+                            extras: [],
+                            version_or_url: Some(
+                                VersionSpecifier(
+                                    VersionSpecifiers(
+                                        [
+                                            VersionSpecifier {
+                                                operator: Equal,
+                                                version: "2.26.0",
+                                            },
+                                        ],
+                                    ),
+                                ),
+                            ),
+                            marker: None,
+                        },
+                    ),
+                    hashes: [
+                        "sha256:fedcba0987654321fedcba0987654321fedcba0987654321fedcba0987654321",
+                    ],
+                },
+                RequirementEntry {
+                    requirement: Named(
+                        Requirement {
+                            name: PackageName(
+                                "black",
+                            ),
+                            extras: [],
+                            version_or_url: Some(
+                                VersionSpecifier(
+                                    VersionSpecifiers(
+                                        [
+                                            VersionSpecifier {
+                                                operator: Equal,
+                                                version: "21.12b0",
+                                            },
+                                        ],
+                                    ),
+                                ),
+                            ),
+                            marker: None,
+                        },
+                    ),
+                    hashes: [],
+                },
+                RequirementEntry {
+                    requirement: Named(
+                        Requirement {
+                            name: PackageName(
+                                "mypy",
+                            ),
+                            extras: [],
+                            version_or_url: Some(
+                                VersionSpecifier(
+                                    VersionSpecifiers(
+                                        [
+                                            VersionSpecifier {
+                                                operator: Equal,
+                                                version: "0.910",
+                                            },
+                                        ],
+                                    ),
+                                ),
+                            ),
+                            marker: None,
+                        },
+                    ),
+                    hashes: [],
+                },
+            ],
+            constraints: [],
+            editables: [],
+            index_url: Some(
+                VerbatimUrl {
+                    url: Url {
+                        scheme: "https",
+                        cannot_be_a_base: false,
+                        username: "",
+                        password: None,
+                        host: Some(
+                            Domain(
+                                "test.pypi.org",
+                            ),
+                        ),
+                        port: None,
+                        path: "/simple/",
+                        query: None,
+                        fragment: None,
+                    },
+                    given: Some(
+                        "https://test.pypi.org/simple/",
+                    ),
+                },
+            ),
+            extra_index_urls: [],
+            find_links: [],
+            no_index: false,
+            no_binary: All,
+            only_binary: None,
+        }
+        "###);
+
+        Ok(())
+    }
+
     #[test]
     fn editable_extra() {
         assert_eq!(
@@ -1957,7 +2178,7 @@ mod test {
         let requirements_txt = temp_dir.child("requirements.txt");
         requirements_txt.write_str(indoc! {"
             numpy>=1,<2
-              --borken
+              --broken
             tqdm
         "})?;
 
@@ -1986,15 +2207,15 @@ mod test {
         Ok(())
     }
 
-    #[test_case("numpy>=1,<2\n  @-borken\ntqdm", "2:4"; "ASCII Character with LF")]
-    #[test_case("numpy>=1,<2\r\n  #-borken\ntqdm", "2:4"; "ASCII Character with CRLF")]
-    #[test_case("numpy>=1,<2\n  \n-borken\ntqdm", "3:1"; "ASCII Character LF then LF")]
-    #[test_case("numpy>=1,<2\n  \r-borken\ntqdm", "3:1"; "ASCII Character LF then CR but no LF")]
-    #[test_case("numpy>=1,<2\n  \r\n-borken\ntqdm", "3:1"; "ASCII Character LF then CRLF")]
-    #[test_case("numpy>=1,<2\n  🚀-borken\ntqdm", "2:4"; "Emoji (Wide) Character")]
-    #[test_case("numpy>=1,<2\n  中-borken\ntqdm", "2:4"; "Fullwidth character")]
-    #[test_case("numpy>=1,<2\n  e\u{0301}-borken\ntqdm", "2:5"; "Two codepoints")]
-    #[test_case("numpy>=1,<2\n  a\u{0300}\u{0316}-borken\ntqdm", "2:6"; "Three codepoints")]
+    #[test_case("numpy>=1,<2\n  @-broken\ntqdm", "2:4"; "ASCII Character with LF")]
+    #[test_case("numpy>=1,<2\r\n  #-broken\ntqdm", "2:4"; "ASCII Character with CRLF")]
+    #[test_case("numpy>=1,<2\n  \n-broken\ntqdm", "3:1"; "ASCII Character LF then LF")]
+    #[test_case("numpy>=1,<2\n  \r-broken\ntqdm", "3:1"; "ASCII Character LF then CR but no LF")]
+    #[test_case("numpy>=1,<2\n  \r\n-broken\ntqdm", "3:1"; "ASCII Character LF then CRLF")]
+    #[test_case("numpy>=1,<2\n  🚀-broken\ntqdm", "2:4"; "Emoji (Wide) Character")]
+    #[test_case("numpy>=1,<2\n  中-broken\ntqdm", "2:4"; "Fullwidth character")]
+    #[test_case("numpy>=1,<2\n  e\u{0301}-broken\ntqdm", "2:5"; "Two codepoints")]
+    #[test_case("numpy>=1,<2\n  a\u{0300}\u{0316}-broken\ntqdm", "2:6"; "Three codepoints")]
     fn test_calculate_line_column_pair(input: &str, expected: &str) {
         let mut s = Scanner::new(input);
         // Place cursor right after the character we want to test
