@@ -7,8 +7,8 @@ use std::fmt::{Display, Formatter};
 use std::io;
 use std::path::{Path, PathBuf};
 use std::process::{ExitStatus, Output};
+use std::rc::Rc;
 use std::str::FromStr;
-use std::sync::Arc;
 use std::{env, iter};
 
 use fs_err as fs;
@@ -25,9 +25,9 @@ use tokio::process::Command;
 use tokio::sync::Mutex;
 use tracing::{debug, info_span, instrument, Instrument};
 
-use distribution_types::Resolution;
+use distribution_types::{ParsedUrlError, Requirement, Resolution};
 use pep440_rs::Version;
-use pep508_rs::{PackageName, Requirement};
+use pep508_rs::PackageName;
 use uv_configuration::{BuildKind, ConfigSettings, SetupPyStrategy};
 use uv_fs::{PythonExt, Simplified};
 use uv_interpreter::{Interpreter, PythonEnvironment};
@@ -54,14 +54,18 @@ static WHEEL_NOT_FOUND_RE: Lazy<Regex> =
 static DEFAULT_BACKEND: Lazy<Pep517Backend> = Lazy::new(|| Pep517Backend {
     backend: "setuptools.build_meta:__legacy__".to_string(),
     backend_path: None,
-    requirements: vec![Requirement::from_str("setuptools >= 40.8.0").unwrap()],
+    requirements: vec![Requirement::from_pep508(
+        pep508_rs::Requirement::from_str("setuptools >= 40.8.0").unwrap(),
+    )
+    .unwrap()],
 });
 
 /// The requirements for `--legacy-setup-py` builds.
 static SETUP_PY_REQUIREMENTS: Lazy<[Requirement; 2]> = Lazy::new(|| {
     [
-        Requirement::from_str("setuptools >= 40.8.0").unwrap(),
-        Requirement::from_str("wheel").unwrap(),
+        Requirement::from_pep508(pep508_rs::Requirement::from_str("setuptools >= 40.8.0").unwrap())
+            .unwrap(),
+        Requirement::from_pep508(pep508_rs::Requirement::from_str("wheel").unwrap()).unwrap(),
     ]
 });
 
@@ -100,6 +104,8 @@ pub enum Error {
     },
     #[error("Failed to build PATH for build script")]
     BuildScriptPath(#[source] env::JoinPathsError),
+    #[error("Failed to parse requirements from build backend")]
+    DirectUrl(#[source] Box<ParsedUrlError>),
 }
 
 #[derive(Debug)]
@@ -223,7 +229,7 @@ pub struct Project {
 #[serde(rename_all = "kebab-case")]
 pub struct BuildSystem {
     /// PEP 508 dependencies required to execute the build system.
-    pub requires: Vec<Requirement>,
+    pub requires: Vec<pep508_rs::Requirement>,
     /// A string naming a Python object that will be used to perform the build.
     pub build_backend: Option<String>,
     /// Specify that their backend code is hosted in-tree, this key contains a list of directories.
@@ -329,13 +335,13 @@ impl Pep517Backend {
     }
 }
 
-/// Uses an [`Arc`] internally, clone freely.
+/// Uses an [`Rc`] internally, clone freely.
 #[derive(Debug, Default, Clone)]
 pub struct SourceBuildContext {
     /// An in-memory resolution of the default backend's requirements for PEP 517 builds.
-    default_resolution: Arc<Mutex<Option<Resolution>>>,
+    default_resolution: Rc<Mutex<Option<Resolution>>>,
     /// An in-memory resolution of the build requirements for `--legacy-setup-py` builds.
-    setup_py_resolution: Arc<Mutex<Option<Resolution>>>,
+    setup_py_resolution: Rc<Mutex<Option<Resolution>>>,
 }
 
 /// Holds the state through a series of PEP 517 frontend to backend calls or a single setup.py
@@ -355,12 +361,12 @@ pub struct SourceBuild {
     venv: PythonEnvironment,
     /// Populated if `prepare_metadata_for_build_wheel` was called.
     ///
-    /// > If the build frontend has previously called prepare_metadata_for_build_wheel and depends
+    /// > If the build frontend has previously called `prepare_metadata_for_build_wheel` and depends
     /// > on the wheel resulting from this call to have metadata matching this earlier call, then
-    /// > it should provide the path to the created .dist-info directory as the metadata_directory
-    /// > argument. If this argument is provided, then build_wheel MUST produce a wheel with
+    /// > it should provide the path to the created .dist-info directory as the `metadata_directory`
+    /// > argument. If this argument is provided, then `build_wheel` MUST produce a wheel with
     /// > identical metadata. The directory passed in by the build frontend MUST be identical to the
-    /// > directory created by prepare_metadata_for_build_wheel, including any unrecognized files
+    /// > directory created by `prepare_metadata_for_build_wheel`, including any unrecognized files
     /// > it created.
     metadata_directory: Option<PathBuf>,
     /// Package id such as `foo-1.2.3`, for error reporting
@@ -414,6 +420,7 @@ impl SourceBuild {
                 interpreter.clone(),
                 uv_virtualenv::Prompt::None,
                 false,
+                false,
             )?,
             BuildIsolation::Shared(venv) => venv.clone(),
         };
@@ -570,7 +577,12 @@ impl SourceBuild {
                             .build_backend
                             .unwrap_or_else(|| "setuptools.build_meta:__legacy__".to_string()),
                         backend_path: build_system.backend_path,
-                        requirements: build_system.requires,
+                        requirements: build_system
+                            .requires
+                            .into_iter()
+                            .map(Requirement::from_pep508)
+                            .collect::<Result<_, _>>()
+                            .map_err(|err| Box::new(Error::DirectUrl(err)))?,
                     }
                 } else {
                     // If a `pyproject.toml` is present, but `[build-system]` is missing, proceed with
@@ -942,7 +954,7 @@ async fn create_pep517_build_environment(
     })?;
 
     // Deserialize the requirements from the output file.
-    let extra_requires: Vec<Requirement> = serde_json::from_slice(&contents).map_err(|err| {
+    let extra_requires: Vec<pep508_rs::Requirement> = serde_json::from_slice::<Vec<pep508_rs::Requirement>>(&contents).map_err(|err| {
         Error::from_command_output(
             format!(
                 "Build backend failed to return extra requires with `get_requires_for_build_{build_kind}`: {err}"
@@ -951,6 +963,11 @@ async fn create_pep517_build_environment(
             version_id,
         )
     })?;
+    let extra_requires: Vec<_> = extra_requires
+        .into_iter()
+        .map(Requirement::from_pep508)
+        .collect::<Result<_, _>>()
+        .map_err(Error::DirectUrl)?;
 
     // Some packages (such as tqdm 4.66.1) list only extra requires that have already been part of
     // the pyproject.toml requires (in this case, `wheel`). We can skip doing the whole resolution
@@ -961,7 +978,7 @@ async fn create_pep517_build_environment(
         .any(|req| !pep517_backend.requirements.contains(req))
     {
         debug!("Installing extra requirements for build backend");
-        let requirements: Vec<Requirement> = pep517_backend
+        let requirements: Vec<_> = pep517_backend
             .requirements
             .iter()
             .cloned()
