@@ -3,14 +3,12 @@ use std::path::PathBuf;
 
 use anstream::println;
 use anyhow::{Context, Result};
-
 use clap::{Parser, ValueEnum};
 use fs_err::File;
 use itertools::Itertools;
 use petgraph::dot::{Config as DotConfig, Dot};
 
-use distribution_types::{FlatIndexLocation, IndexLocations, IndexUrl, Resolution};
-use pep508_rs::Requirement;
+use distribution_types::{FlatIndexLocation, IndexLocations, IndexUrl, Requirement, Resolution};
 use uv_cache::{Cache, CacheArgs};
 use uv_client::{FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{ConfigSettings, NoBinary, NoBuild, SetupPyStrategy};
@@ -29,7 +27,7 @@ pub(crate) enum ResolveCliFormat {
 
 #[derive(Parser)]
 pub(crate) struct ResolveCliArgs {
-    requirements: Vec<Requirement>,
+    requirements: Vec<pep508_rs::Requirement>,
     /// Write debug output in DOT format for graphviz to this file
     #[clap(long)]
     graphviz: Option<PathBuf>,
@@ -101,7 +99,13 @@ pub(crate) async fn resolve_cli(args: ResolveCliArgs) -> Result<()> {
     // Copied from `BuildDispatch`
     let tags = venv.interpreter().tags()?;
     let resolver = Resolver::new(
-        Manifest::simple(args.requirements.clone()),
+        Manifest::simple(
+            args.requirements
+                .iter()
+                .cloned()
+                .map(Requirement::from_pep508)
+                .collect::<Result<_, _>>()?,
+        ),
         Options::default(),
         venv.interpreter().markers(),
         venv.interpreter(),
