@@ -16,10 +16,8 @@ use platform_tags::{Tags, TagsError};
 use pypi_types::Scheme;
 use uv_cache::{Cache, CacheBucket, CachedByTimestamp, Freshness, Timestamp};
 use uv_fs::{write_atomic_sync, PythonExt, Simplified};
-use uv_toolchain::PythonVersion;
 
-use crate::Virtualenv;
-use crate::{Error, Target};
+use crate::{Error, PythonVersion, Target, VirtualEnvironment};
 
 /// A Python executable and its associated platform markers.
 #[derive(Debug, Clone)]
@@ -100,11 +98,12 @@ impl Interpreter {
 
     /// Return a new [`Interpreter`] with the given virtual environment root.
     #[must_use]
-    pub fn with_virtualenv(self, virtualenv: Virtualenv) -> Self {
+    pub fn with_virtualenv(self, virtualenv: VirtualEnvironment) -> Self {
         Self {
             scheme: virtualenv.scheme,
             sys_executable: virtualenv.executable,
             prefix: virtualenv.root,
+            target: None,
             ..self
         }
     }
@@ -405,8 +404,8 @@ pub enum InterpreterInfoError {
     LibcNotFound,
     #[error("Unknown operation system: `{operating_system}`")]
     UnknownOperatingSystem { operating_system: String },
-    #[error("Python 2 is not supported. Please use Python 3.8 or newer.")]
-    UnsupportedPythonVersion,
+    #[error("Python {python_version} is not supported. Please use Python 3.8 or newer.")]
+    UnsupportedPythonVersion { python_version: String },
 }
 
 #[derive(Debug, Deserialize, Serialize, Clone)]
