@@ -6,7 +6,7 @@ pub use crate::unnamed::*;
 
 mod confirm;
 mod lookahead;
-mod pyproject;
+pub mod pyproject;
 mod source_tree;
 mod sources;
 mod specification;
