@@ -11,9 +11,9 @@ use uv_cache::CacheArgs;
 use uv_configuration::{
     ConfigSettingEntry, IndexStrategy, KeyringProviderType, PackageNameSpecifier, TargetTriple,
 };
+use uv_interpreter::PythonVersion;
 use uv_normalize::{ExtraName, PackageName};
 use uv_resolver::{AnnotationStyle, ExcludeNewer, PreReleaseMode, ResolutionMode};
-use uv_toolchain::PythonVersion;
 
 use crate::commands::{extra_name_with_clap_error, ListFormat, VersionFormat};
 use crate::compat;
@@ -370,9 +370,12 @@ pub(crate) struct PipCompileArgs {
     pub(crate) no_offline: bool,
 
     /// Refresh all cached data.
-    #[arg(long)]
+    #[arg(long, overrides_with("no_refresh"))]
     pub(crate) refresh: bool,
 
+    #[arg(long, overrides_with("refresh"), hide = true)]
+    pub(crate) no_refresh: bool,
+
     /// Refresh cached data for a specific package.
     #[arg(long)]
     pub(crate) refresh_package: Vec<PackageName>,
@@ -383,7 +386,7 @@ pub(crate) struct PipCompileArgs {
     ///
     /// Defaults to `clone` (also known as Copy-on-Write) on macOS, and `hardlink` on Linux and
     /// Windows.
-    #[arg(long, value_enum)]
+    #[arg(long, value_enum, env = "UV_LINK_MODE")]
     pub(crate) link_mode: Option<install_wheel_rs::linker::LinkMode>,
 
     /// The URL of the Python package index (by default: <https://pypi.org/simple>).
@@ -474,9 +477,12 @@ pub(crate) struct PipCompileArgs {
     pub(crate) no_system: bool,
 
     /// Allow package upgrades, ignoring pinned versions in the existing output file.
-    #[arg(long, short = 'U')]
+    #[arg(long, short = 'U', overrides_with("no_upgrade"))]
     pub(crate) upgrade: bool,
 
+    #[arg(long, overrides_with("upgrade"), hide = true)]
+    pub(crate) no_upgrade: bool,
+
     /// Allow upgrades for a specific package, ignoring pinned versions in the existing output
     /// file.
     #[arg(long, short = 'P')]
@@ -500,7 +506,12 @@ pub(crate) struct PipCompileArgs {
     /// Disable isolation when building source distributions.
     ///
     /// Assumes that build dependencies specified by PEP 518 are already installed.
-    #[arg(long, overrides_with("build_isolation"))]
+    #[arg(
+        long,
+        env = "UV_NO_BUILD_ISOLATION",
+        value_parser = clap::builder::BoolishValueParser::new(),
+        overrides_with("build_isolation")
+    )]
     pub(crate) no_build_isolation: bool,
 
     #[arg(long, overrides_with("no_build_isolation"), hide = true)]
@@ -601,6 +612,12 @@ pub(crate) struct PipCompileArgs {
     #[arg(long, overrides_with("emit_index_annotation"), hide = true)]
     pub(crate) no_emit_index_annotation: bool,
 
+    #[arg(long, overrides_with("no_unstable_uv_lock_file"), hide = true)]
+    pub(crate) unstable_uv_lock_file: bool,
+
+    #[arg(long, overrides_with("unstable_uv_lock_file"), hide = true)]
+    pub(crate) no_unstable_uv_lock_file: bool,
+
     #[command(flatten)]
     pub(crate) compat_args: compat::PipCompileCompatArgs,
 }
@@ -613,9 +630,12 @@ pub(crate) struct PipSyncArgs {
     pub(crate) src_file: Vec<PathBuf>,
 
     /// Reinstall all packages, regardless of whether they're already installed.
-    #[arg(long, alias = "force-reinstall")]
+    #[arg(long, alias = "force-reinstall", overrides_with("no_reinstall"))]
     pub(crate) reinstall: bool,
 
+    #[arg(long, overrides_with("reinstall"), hide = true)]
+    pub(crate) no_reinstall: bool,
+
     /// Reinstall a specific package, regardless of whether it's already installed.
     #[arg(long)]
     pub(crate) reinstall_package: Vec<PackageName>,
@@ -633,9 +653,12 @@ pub(crate) struct PipSyncArgs {
     pub(crate) no_offline: bool,
 
     /// Refresh all cached data.
-    #[arg(long)]
+    #[arg(long, overrides_with("no_refresh"))]
     pub(crate) refresh: bool,
 
+    #[arg(long, overrides_with("refresh"), hide = true)]
+    pub(crate) no_refresh: bool,
+
     /// Refresh cached data for a specific package.
     #[arg(long)]
     pub(crate) refresh_package: Vec<PackageName>,
@@ -644,7 +667,7 @@ pub(crate) struct PipSyncArgs {
     ///
     /// Defaults to `clone` (also known as Copy-on-Write) on macOS, and `hardlink` on Linux and
     /// Windows.
-    #[arg(long, value_enum)]
+    #[arg(long, value_enum, env = "UV_LINK_MODE")]
     pub(crate) link_mode: Option<install_wheel_rs::linker::LinkMode>,
 
     /// The URL of the Python package index (by default: <https://pypi.org/simple>).
@@ -732,7 +755,13 @@ pub(crate) struct PipSyncArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(long, short, verbatim_doc_comment, group = "discovery")]
+    #[arg(
+        long,
+        short,
+        env = "UV_PYTHON",
+        verbatim_doc_comment,
+        group = "discovery"
+    )]
     pub(crate) python: Option<String>,
 
     /// Install packages into the system Python.
@@ -789,7 +818,13 @@ pub(crate) struct PipSyncArgs {
     /// Disable isolation when building source distributions.
     ///
     /// Assumes that build dependencies specified by PEP 518 are already installed.
-    #[arg(long, overrides_with("build_isolation"))]
+
+    #[arg(
+        long,
+        env = "UV_NO_BUILD_ISOLATION",
+        value_parser = clap::builder::BoolishValueParser::new(),
+        overrides_with("build_isolation")
+    )]
     pub(crate) no_build_isolation: bool,
 
     #[arg(long, overrides_with("no_build_isolation"), hide = true)]
@@ -948,17 +983,23 @@ pub(crate) struct PipInstallArgs {
     pub(crate) no_all_extras: bool,
 
     /// Allow package upgrades.
-    #[arg(long, short = 'U')]
+    #[arg(long, short = 'U', overrides_with("no_upgrade"))]
     pub(crate) upgrade: bool,
 
+    #[arg(long, overrides_with("upgrade"), hide = true)]
+    pub(crate) no_upgrade: bool,
+
     /// Allow upgrade of a specific package.
     #[arg(long, short = 'P')]
     pub(crate) upgrade_package: Vec<PackageName>,
 
     /// Reinstall all packages, regardless of whether they're already installed.
-    #[arg(long, alias = "force-reinstall")]
+    #[arg(long, alias = "force-reinstall", overrides_with("no_reinstall"))]
     pub(crate) reinstall: bool,
 
+    #[arg(long, overrides_with("reinstall"), hide = true)]
+    pub(crate) no_reinstall: bool,
+
     /// Reinstall a specific package, regardless of whether it's already installed.
     #[arg(long)]
     pub(crate) reinstall_package: Vec<PackageName>,
@@ -976,9 +1017,12 @@ pub(crate) struct PipInstallArgs {
     pub(crate) no_offline: bool,
 
     /// Refresh all cached data.
-    #[arg(long)]
+    #[arg(long, overrides_with("no_refresh"))]
     pub(crate) refresh: bool,
 
+    #[arg(long, overrides_with("refresh"), hide = true)]
+    pub(crate) no_refresh: bool,
+
     /// Refresh cached data for a specific package.
     #[arg(long)]
     pub(crate) refresh_package: Vec<PackageName>,
@@ -995,7 +1039,7 @@ pub(crate) struct PipInstallArgs {
     ///
     /// Defaults to `clone` (also known as Copy-on-Write) on macOS, and `hardlink` on Linux and
     /// Windows.
-    #[arg(long, value_enum)]
+    #[arg(long, value_enum, env = "UV_LINK_MODE")]
     pub(crate) link_mode: Option<install_wheel_rs::linker::LinkMode>,
 
     /// The strategy to use when selecting between the different compatible versions for a given
@@ -1101,7 +1145,13 @@ pub(crate) struct PipInstallArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(long, short, verbatim_doc_comment, group = "discovery")]
+    #[arg(
+        long,
+        short,
+        env = "UV_PYTHON",
+        verbatim_doc_comment,
+        group = "discovery"
+    )]
     pub(crate) python: Option<String>,
 
     /// Install packages into the system Python.
@@ -1158,7 +1208,12 @@ pub(crate) struct PipInstallArgs {
     /// Disable isolation when building source distributions.
     ///
     /// Assumes that build dependencies specified by PEP 518 are already installed.
-    #[arg(long, overrides_with("build_isolation"))]
+    #[arg(
+        long,
+        env = "UV_NO_BUILD_ISOLATION",
+        value_parser = clap::builder::BoolishValueParser::new(),
+        overrides_with("build_isolation")
+    )]
     pub(crate) no_build_isolation: bool,
 
     #[arg(long, overrides_with("no_build_isolation"), hide = true)]
@@ -1274,6 +1329,12 @@ pub(crate) struct PipInstallArgs {
     /// print the resulting plan.
     #[arg(long)]
     pub(crate) dry_run: bool,
+
+    #[arg(long, hide = true)]
+    pub(crate) unstable_uv_lock_file: Option<String>,
+
+    #[command(flatten)]
+    pub(crate) compat_args: compat::PipInstallCompatArgs,
 }
 
 #[derive(Args)]
@@ -1300,7 +1361,13 @@ pub(crate) struct PipUninstallArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(long, short, verbatim_doc_comment, group = "discovery")]
+    #[arg(
+        long,
+        short,
+        env = "UV_PYTHON",
+        verbatim_doc_comment,
+        group = "discovery"
+    )]
     pub(crate) python: Option<String>,
 
     /// Attempt to use `keyring` for authentication for remote requirements files.
@@ -1389,7 +1456,13 @@ pub(crate) struct PipFreezeArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(long, short, verbatim_doc_comment, group = "discovery")]
+    #[arg(
+        long,
+        short,
+        env = "UV_PYTHON",
+        verbatim_doc_comment,
+        group = "discovery"
+    )]
     pub(crate) python: Option<String>,
 
     /// List packages for the system Python.
@@ -1452,7 +1525,13 @@ pub(crate) struct PipListArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(long, short, verbatim_doc_comment, group = "discovery")]
+    #[arg(
+        long,
+        short,
+        env = "UV_PYTHON",
+        verbatim_doc_comment,
+        group = "discovery"
+    )]
     pub(crate) python: Option<String>,
 
     /// List packages for the system Python.
@@ -1494,7 +1573,13 @@ pub(crate) struct PipCheckArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(long, short, verbatim_doc_comment, group = "discovery")]
+    #[arg(
+        long,
+        short,
+        env = "UV_PYTHON",
+        verbatim_doc_comment,
+        group = "discovery"
+    )]
     pub(crate) python: Option<String>,
 
     /// List packages for the system Python.
@@ -1544,7 +1629,13 @@ pub(crate) struct PipShowArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(long, short, verbatim_doc_comment, group = "discovery")]
+    #[arg(
+        long,
+        short,
+        env = "UV_PYTHON",
+        verbatim_doc_comment,
+        group = "discovery"
+    )]
     pub(crate) python: Option<String>,
 
     /// List packages for the system Python.
@@ -1582,7 +1673,13 @@ pub(crate) struct VenvArgs {
     ///
     /// Note that this is different from `--python-version` in `pip compile`, which takes `3.10` or `3.10.13` and
     /// doesn't look for a Python interpreter on disk.
-    #[arg(long, short, verbatim_doc_comment, group = "discovery")]
+    #[arg(
+        long,
+        short,
+        env = "UV_PYTHON",
+        verbatim_doc_comment,
+        group = "discovery"
+    )]
     pub(crate) python: Option<String>,
 
     /// Use the system Python to uninstall packages.
@@ -1609,6 +1706,18 @@ pub(crate) struct VenvArgs {
     #[arg(long)]
     pub(crate) seed: bool,
 
+    /// Preserve any existing files or directories at the target path.
+    ///
+    /// By default, `uv venv` will remove an existing virtual environment at the given path, and
+    /// exit with an error if the path is non-empty but _not_ a virtual environment. The
+    /// `--allow-existing` option will instead write to the given path, regardless of its contents,
+    /// and without clearing it beforehand.
+    ///
+    /// WARNING: This option can lead to unexpected behavior if the existing virtual environment
+    /// and the newly-created virtual environment are linked to different Python interpreters.
+    #[clap(long)]
+    pub(crate) allow_existing: bool,
+
     /// The path to the virtual environment to create.
     #[arg(default_value = ".venv")]
     pub(crate) name: PathBuf,
@@ -1641,7 +1750,7 @@ pub(crate) struct VenvArgs {
     ///
     /// Defaults to `clone` (also known as Copy-on-Write) on macOS, and `hardlink` on Linux and
     /// Windows.
-    #[arg(long, value_enum)]
+    #[arg(long, value_enum, env = "UV_LINK_MODE")]
     pub(crate) link_mode: Option<install_wheel_rs::linker::LinkMode>,
 
     /// The URL of the Python package index (by default: <https://pypi.org/simple>).
@@ -1718,10 +1827,6 @@ pub(crate) struct RunArgs {
     #[arg(allow_hyphen_values = true)]
     pub(crate) args: Vec<OsString>,
 
-    /// Always use a new virtual environment for execution.
-    #[arg(long)]
-    pub(crate) isolated: bool,
-
     /// Run with the given packages installed.
     #[arg(long)]
     pub(crate) with: Vec<String>,
@@ -1737,13 +1842,14 @@ pub(crate) struct RunArgs {
     ///   `python3.10` on Linux and macOS.
     /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
-    #[arg(long, short, verbatim_doc_comment, group = "discovery")]
+    #[arg(
+        long,
+        short,
+        env = "UV_PYTHON",
+        verbatim_doc_comment,
+        group = "discovery"
+    )]
     pub(crate) python: Option<String>,
-
-    /// Run without the current workspace installed.
-    #[arg(long)]
-    pub(crate) no_workspace: bool,
-    // TODO(zanieb): Consider alternative names like `--no-project` or `--without-project`
 }
 
 #[derive(Args)]
