@@ -1,3 +1,4 @@
+use indexmap::IndexMap;
 use std::borrow::Cow;
 use std::env;
 use std::fmt::Write;
@@ -8,26 +9,31 @@ use std::str::FromStr;
 
 use anstream::{eprint, AutoStream, StripStream};
 use anyhow::{anyhow, Context, Result};
+use fs_err as fs;
 use itertools::Itertools;
 use owo_colors::OwoColorize;
 use tempfile::tempdir_in;
 use tracing::debug;
 
-use distribution_types::{IndexLocations, LocalEditable, LocalEditables, Verbatim};
+use distribution_types::{IndexLocations, LocalEditable, LocalEditables, ParsedUrlError, Verbatim};
+use distribution_types::{Requirement, Requirements};
 use install_wheel_rs::linker::LinkMode;
+
 use platform_tags::Tags;
+use pypi_types::Metadata23;
 use requirements_txt::EditableRequirement;
 use uv_auth::store_credentials_from_url;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{
-    ConfigSettings, Constraints, IndexStrategy, NoBinary, NoBuild, Overrides, SetupPyStrategy,
-    Upgrade,
+    ConfigSettings, Constraints, IndexStrategy, NoBinary, NoBuild, Overrides, PreviewMode,
+    SetupPyStrategy, Upgrade,
 };
 use uv_configuration::{KeyringProviderType, TargetTriple};
 use uv_dispatch::BuildDispatch;
 use uv_fs::Simplified;
 use uv_installer::Downloader;
+use uv_interpreter::PythonVersion;
 use uv_interpreter::{find_best_python, find_requested_python, PythonEnvironment};
 use uv_normalize::{ExtraName, PackageName};
 use uv_requirements::{
@@ -39,7 +45,6 @@ use uv_resolver::{
     InMemoryIndex, Manifest, OptionsBuilder, PreReleaseMode, PythonRequirement, ResolutionMode,
     Resolver,
 };
-use uv_toolchain::PythonVersion;
 use uv_types::{BuildIsolation, EmptyInstalledPackages, HashStrategy, InFlight};
 use uv_warnings::warn_user;
 
@@ -84,8 +89,10 @@ pub(crate) async fn pip_compile(
     link_mode: LinkMode,
     python: Option<String>,
     system: bool,
+    uv_lock: bool,
     native_tls: bool,
     quiet: bool,
+    preview: PreviewMode,
     cache: Cache,
     printer: Printer,
 ) -> Result<ExitStatus> {
@@ -125,6 +132,7 @@ pub(crate) async fn pip_compile(
         overrides,
         &extras,
         &client_builder,
+        preview,
     )
     .await?;
 
@@ -364,17 +372,33 @@ pub(crate) async fn pip_compile(
 
         // Build all editables.
         let editable_wheel_dir = tempdir_in(cache.root())?;
-        let editables: Vec<_> = downloader
+        let editables: Vec<(LocalEditable, Metadata23, Requirements)> = downloader
             .build_editables(editables, editable_wheel_dir.path())
             .await
             .context("Failed to build editables")?
             .into_iter()
-            .map(|built_editable| (built_editable.editable, built_editable.metadata))
-            .collect();
+            .map(|built_editable| {
+                let requirements = Requirements {
+                    dependencies: built_editable
+                        .metadata
+                        .requires_dist
+                        .iter()
+                        .cloned()
+                        .map(Requirement::from_pep508)
+                        .collect::<Result<_, _>>()?,
+                    optional_dependencies: IndexMap::default(),
+                };
+                Ok::<_, Box<ParsedUrlError>>((
+                    built_editable.editable,
+                    built_editable.metadata,
+                    requirements,
+                ))
+            })
+            .collect::<Result<_, _>>()?;
 
         // Validate that the editables are compatible with the target Python version.
         let requirement = PythonRequirement::new(&interpreter, &markers);
-        for (.., metadata) in &editables {
+        for (_, metadata, _) in &editables {
             if let Some(python_requires) = metadata.requires_python.as_ref() {
                 if !python_requires.contains(requirement.target()) {
                     return Err(anyhow!(
@@ -516,7 +540,7 @@ pub(crate) async fn pip_compile(
     }
 
     if include_marker_expression {
-        let relevant_markers = resolution.marker_tree(&manifest, &top_level_index, &markers);
+        let relevant_markers = resolution.marker_tree(&manifest, &top_level_index, &markers)?;
         writeln!(
             writer,
             "{}",
@@ -525,6 +549,12 @@ pub(crate) async fn pip_compile(
         writeln!(writer, "{}", format!("#    {relevant_markers}").green())?;
     }
 
+    if uv_lock {
+        let lock = resolution.lock()?;
+        let encoded = toml::to_string_pretty(&lock)?;
+        fs::tokio::write("uv.lock", encoded.as_bytes()).await?;
+    }
+
     // Write the index locations to the output channel.
     let mut wrote_index = false;
 
